﻿// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.
// See the LICENSE file in the project root for more information.

using System.Linq;
using Microsoft.CodeAnalysis.CSharp.LanguageService;
using Microsoft.CodeAnalysis.CSharp.Syntax;
using Microsoft.CodeAnalysis.Diagnostics;
using Microsoft.CodeAnalysis.LanguageService;
using Microsoft.CodeAnalysis.ValidateFormatString;

namespace Microsoft.CodeAnalysis.CSharp.ValidateFormatString;

[DiagnosticAnalyzer(LanguageNames.CSharp)]
internal class CSharpValidateFormatStringDiagnosticAnalyzer :
    AbstractValidateFormatStringDiagnosticAnalyzer<SyntaxKind>
{
    protected override ISyntaxFacts GetSyntaxFacts()
        => CSharpSyntaxFacts.Instance;

    protected override SyntaxNode? TryGetMatchingNamedArgument(
        SeparatedSyntaxList<SyntaxNode> arguments,
        string searchArgumentName)
    {
        foreach (var argument in arguments.Cast<ArgumentSyntax>())
        {
            if (argument.NameColon != null && argument.NameColon.Name.Identifier.ValueText.Equals(searchArgumentName))
            {
                return argument;
            }
        }

        return null;
    }

    protected override SyntaxNode GetArgumentExpression(SyntaxNode syntaxNode)
        => ((ArgumentSyntax)syntaxNode).Expression;
}
